# Electronic Supplemental Material for: 

# Reum, JCP, Holsman K, Aydin K, Blanchard B, Jennings S. In Revision. Energetically relevant predator-prey 
#       body mass ratios and their relationship with predator body size. Ecology and Evolution 


# Please the above citation for details regarding the model and data preprocessing. 
# Consult associated README document for descriptions of the variables contained in the data file "ppmrData_AK.csv"


# Before stepping through the script, download the csv file "ppmrData_AL.csv" and 
# set the R working directory to the file location. Okay, let's get started. 

# WARNING: Actual fitting of the model takes ~10 hours on a 2.7 GHz Intel Core i5 MacBook pro machine.

#Load necessary libraries 
library(latticeExtra)   # For plotting
library(brms)           # For fitting the hierarchical model 


#Load up PPMR data, consult associated README.doc for variable descriptions. 
ppmr<-read.csv("ppmrData_AK.csv", row.names=1)


#Set thinning rate, number of chains, warmup, and total number of iterations for MCMC 
thin=3
chains=3
warmup=50
iter=round(warmup + (1500*thin)/chains)


system.time(
modB2<- brm(formula = logppmrb ~ predwt1b + predwt2b +
            (1|predstrat) +
            (1|predreg) +
            (1|common_name) +
            (1|Family) +
            (1|Order) +
            (0 + predwt1b|predstrat) +
            (0 + predwt1b|predreg) +
            (0 + predwt1b|common_name) +
            (0 + predwt1b|Family) +
            (0 + predwt1b|Order) +
            (0 + predwt2b|predstrat) +
            (0 + predwt2b|predreg) +
            (0 + predwt2b|common_name) +
            (0 + predwt2b|Family) +
            (0 + predwt2b|Order),
            data = test, family = gaussian(),
            prior = c(set_prior("normal(0,3)", class = "b"),
                      set_prior("cauchy(0,3)", class = "sd")),
            warmup = warmup, iter = iter, chains = chains, thin=thin,
            control = list(adapt_delta = .99, max_treedepth=100), cores=chains)
)


#About 10 hours! 
system.time(
modN2<- brm(formula = logppmrn ~ predwt1n + predwt2n +
            (1|predstrat) +
            (1|predreg) +
            (1|common_name) +
            (1|Family) +
            (1|Order) +
            (0 + predwt1n|predstrat) +
            (0 + predwt1n|predreg) +
            (0 + predwt1n|common_name) +
            (0 + predwt1n|Family) +
            (0 + predwt1n|Order) +
            (0 + predwt2n|predstrat) +
            (0 + predwt2n|predreg) +
            (0 + predwt2n|common_name) +
            (0 + predwt2n|Family) +
            (0 + predwt2n|Order),
            data = ppmr, family = gaussian(),
            prior = c(set_prior("normal(0,3)", class = "b"),
                      set_prior("cauchy(0,3)", class = "sd")),
            warmup = warmup, iter = iter, chains = chains, thin=thin,
            control = list(adapt_delta = .99, max_treedepth=100), cores=chains)
)


#save(modN2, file="modelBRMS_N2.Rdata")
#save(modB2, file="modelBRMS_B2.Rdata")


thin=1
chains=3
warmup=50
iter=round(warmup + (500*thin)/chains)

#Check that the model performs well 
pp_check(modB2, type = "error_hist", nsamples = 200)
pp_check(modB2, type = "scatter_avg", nsamples = 200)
pp_check(modB2, type = "stat_2d")

#Check that the model performs well 
pp_check(modN2, type = "error_hist", nsamples = 200)
pp_check(modN2, type = "scatter_avg", nsamples = 200)
pp_check(modN2, type = "stat_2d")


#Get model predictions for each level
ppmrnew<-ppmr

ppmrnew$predbm<-fitted(modB2, re_formula=NA)[,1] #Consider no group level effects
ppmrnew$predb_stratum<-fitted(modB2, re_formula=NULL)[,1] #Consider all group levels 

ppmrnew$prednm<-fitted(modN2, re_formula=NA)[,1] #Consider no group level effects
ppmrnew$predn_stratum<-fitted(modN2, re_formula=NULL)[,1] #Consider all group levels 



#Fix + strat + reg
#ORDER
ppmrnew$predb_ord<-fitted(modB2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1b|Order) +
        (0+predwt2b|Order))[,1]


ppmrnew$predn_ord<-fitted(modN2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1n|Order) +
        (0+predwt2n|Order))[,1]

#FAMILY
ppmrnew$predb_fam<-fitted(modB2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1b|Order) +
        (0+predwt2b|Order) +
        (1|Family) +
        (0+predwt1b|Family) +
        (0+predwt2b|Family))[,1]

ppmrnew$predn_fam<-fitted(modN2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1n|Order) +
        (0+predwt2n|Order) +
        (1|Family) +
        (0+predwt1n|Family) +
        (0+predwt2n|Family))[,1]

#COMMON NAME
ppmrnew$predb_nam<-fitted(modB2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1b|Order) +
        (0+predwt2b|Order) +
        (1|Family) +
        (0+predwt1b|Family) +
        (0+predwt2b|Family) +
        (1|common_name) +
        (0+predwt1b|common_name) +
        (0+predwt2b|common_name))[,1]

ppmrnew$predn_nam<-fitted(modN2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1n|Order) +
        (0+predwt2n|Order) +
        (1|Family) +
        (0+predwt1n|Family) +
        (0+predwt2n|Family) +
        (1|common_name) +
        (0+predwt1n|common_name) +
        (0+predwt2n|common_name))[,1]

#REGION
ppmrnew$predb_regs<-fitted(modB2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1b|Order) +
        (0+predwt2b|Order) +
        (1|Family) +
        (0+predwt1b|Family) +
        (0+predwt2b|Family) +
        (1|common_name) +
        (0+predwt1b|common_name) +
        (0+predwt2b|common_name) +
        (1|predreg) +
        (0+predwt1b|predreg) +
        (0+predwt2b|predreg))[,1]

ppmrnew$predn_regs<-fitted(modN2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1n|Order) +
        (0+predwt2n|Order) +
        (1|Family) +
        (0+predwt1n|Family) +
        (0+predwt2n|Family) +
        (1|common_name) +
        (0+predwt1n|common_name) +
        (0+predwt2n|common_name) +
        (1|predreg) +
        (0+predwt1n|predreg) +
        (0+predwt2n|predreg))[,1]
#STRATUM
ppmrnew$predb_strat<-fitted(modB2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1b|Order) +
        (0+predwt2b|Order) +
        (1|Family) +
        (0+predwt1b|Family) +
        (0+predwt2b|Family) +
        (1|common_name) +
        (0+predwt1b|common_name) +
        (0+predwt2b|common_name) +
        (1|predreg) +
        (0+predwt1b|predreg) +
        (0+predwt2b|predreg) +
        (1|predstrat) +
        (0+predwt1b|predstrat) +
        (0+predwt2b|predstrat))[,1]

ppmrnew$predn_strat<-fitted(modN2,allow_new_levels =FALSE, re_formula= ~ 
        (1|Order) +
        (0+predwt1n|Order) +
        (0+predwt2n|Order) +
        (1|Family) +
        (0+predwt1n|Family) +
        (0+predwt2n|Family) +
        (1|common_name) +
        (0+predwt1n|common_name) +
        (0+predwt2n|common_name) +
        (1|predreg) +
        (0+predwt1n|predreg) +
        (0+predwt2n|predreg) +
        (1|predstrat) +
        (0+predwt1n|predstrat) +
        (0+predwt2n|predstrat))[,1]



####Build plots of model predictions #####

ppmrfixed<-aggregate(cbind(predbm)~ predwtbin ,data=ppmrnew, FUN=mean)
ppmrstat<-aggregate(cbind(predb_strat,predbm)~ predstrat+ predwtbin+predreg+common_name+Family+Order +REGION,data=ppmrnew, FUN=mean)
ppmrreg<-aggregate(cbind(predb_regs,predbm)~predwtbin+predreg+common_name+Family+Order+REGION,data=ppmrnew, FUN=mean)
ppmrnam<-aggregate(cbind(predb_nam,predbm)~predwtbin+common_name+Family+Order+REGION,data=ppmrnew, FUN=mean)
ppmrfam<-aggregate(cbind(predb_fam,predbm)~predwtbin+Family+Order+REGION,data=ppmrnew, FUN=mean)
ppmrord<-aggregate(cbind(predb_ord,predbm)~predwtbin+Order+REGION,data=ppmrnew, FUN=mean)

ppmrfixedn<-aggregate(cbind(prednm)~ predwtbin ,data=ppmrnew, FUN=mean)
ppmrstatn<-aggregate(cbind(predn_strat,prednm)~ predstrat+ predwtbin+predreg+common_name+Family+Order +REGION,data=ppmrnew, FUN=mean)
ppmrregn<-aggregate(cbind(predn_regs,prednm)~predwtbin+predreg+common_name+Family+Order+REGION,data=ppmrnew, FUN=mean)
ppmrnamn<-aggregate(cbind(predn_nam,prednm)~predwtbin+common_name+Family+Order+REGION,data=ppmrnew, FUN=mean)
ppmrfamn<-aggregate(cbind(predn_fam,prednm)~predwtbin+Family+Order+REGION,data=ppmrnew, FUN=mean)
ppmrordn<-aggregate(cbind(predn_ord,prednm)~predwtbin+Order+REGION,data=ppmrnew, FUN=mean)


#################
# STRATUM PLOTS #
#################
pointsize<- .25

p1<-xyplot(predb_strat~log10(predwtbin), group=predstrat, data=ppmrstat, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")

#num
p1<-xyplot(predn_strat~log10(predwtbin), group=predstrat, data=ppmrstatn, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")


################
# REGION PLOTS #
################

p1<-xyplot(predb_regs~log10(predwtbin), group=predreg, data=ppmrreg, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")

#num

p1<-xyplot(predn_regs~log10(predwtbin), group=predreg, data=ppmrregn, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")


#################
# SPECIES PLOTS #
#################

ppmrnam<- ppmrnam[order(ppmrnam$predwtbin),]
p1<-xyplot(predb_nam~log10(predwtbin), group=common_name, data=ppmrnam, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")

#num

ppmrnamn<- ppmrnamn[order(ppmrnamn$predwtbin),]
p1<-xyplot(predn_nam~log10(predwtbin), group=common_name, data=ppmrnamn, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")


#################
# FAMILY PLOTS #
#################

ppmrfam<- ppmrfam[order(ppmrfam$predwtbin),]
p1<-xyplot(predb_fam~log10(predwtbin), group=Family, data=ppmrfam, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")

#Num

ppmrfamn<- ppmrfamn[order(ppmrfamn$predwtbin),]
p1<-xyplot(predn_fam~log10(predwtbin), group=Family, data=ppmrfamn, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")


#################
# Order PLOTS #
#################


#Biomass
pmrord<- ppmrord[order(ppmrord$predwtbin),]
p1<-xyplot(predb_ord~log10(predwtbin), group=Order, data=ppmrord, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")

#Num

ppmrordn<- ppmrordn[order(ppmrordn$predwtbin),]
p1<-xyplot(predn_ord~log10(predwtbin), group=Order, data=ppmrordn, type=c("p","l"),
     ylim=c(.5,5.5), xlim=c(0, 5), col="goldenrod1", lwd=1, pch=19,cex=pointsize,
     scales=list(tck=c(.5,0)), ylab="", xlab="")


############################
# Plot ppmrbio vs. ppmrnum #
############################

p1<-xyplot(log10(ppmrNum)~log10(ppmrBio),data=ppmr, ylim=c(.5,7), xlim=c(.5,7),
         las=1, tck=-.01,pch=19,cex=.25,
         scales=list(tck=c(.5,0)), ylab="", xlab="", col="goldenrod1") 

#code for num

p1<-xyplot(log10(ppmrNum)~log10(ppmrBio),data=ppmr, ylim=c(.5,7), xlim=c(.5,7),
         las=1, tck=-.01,pch=19,cex=.25,
         scales=list(tck=c(.5,0)), ylab="", xlab="", col="goldenrod1") 







































